@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
+** a written agreement between you and Digia. For licensing terms and
+** conditions see http://qt.digia.com/licensing. For further information
 ** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** rights. These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -42,6 +34,7 @@
 #include "qgstutils_p.h"
 
 #include <QtCore/qdatetime.h>
+#include <QtCore/qdir.h>
 #include <QtCore/qbytearray.h>
 #include <QtCore/qvariant.h>
 #include <QtCore/qsize.h>
@@ -49,6 +42,13 @@
 #include <QtCore/qstringlist.h>
 #include <qaudioformat.h>
 
+#ifdef USE_V4L
+#  include <private/qcore_unix_p.h>
+#  include <linux/videodev2.h>
+#endif
+
+#include "qgstreamervideoinputdevicecontrol_p.h"
+
 QT_BEGIN_NAMESPACE
 
 //internal
@@ -401,4 +401,200 @@ QMultimedia::SupportEstimate QGstUtils::hasSupport(const QString &mimeType,
     return QMultimedia::MaybeSupported;
 }
 
+namespace {
+
+typedef QHash<GstElementFactory *, QVector<QGstUtils::CameraInfo> > FactoryCameraInfoMap;
+
+Q_GLOBAL_STATIC(FactoryCameraInfoMap, qt_camera_device_info);
+
+}
+
+QVector<QGstUtils::CameraInfo> QGstUtils::enumerateCameras(GstElementFactory *factory)
+{
+    FactoryCameraInfoMap::const_iterator it = qt_camera_device_info()->constFind(factory);
+    if (it != qt_camera_device_info()->constEnd())
+        return *it;
+
+    QVector<CameraInfo> &devices = (*qt_camera_device_info())[factory];
+
+    if (factory) {
+        bool hasVideoSource = false;
+
+        const GType type = gst_element_factory_get_element_type(factory);
+        GObjectClass * const objectClass = type
+                ? static_cast<GObjectClass *>(g_type_class_ref(type))
+                : 0;
+        if (objectClass) {
+            if (g_object_class_find_property(objectClass, "camera-device")) {
+                const CameraInfo primary = {
+                    QStringLiteral("primary"),
+                    QGstreamerVideoInputDeviceControl::primaryCamera(),
+                    0,
+                    QCamera::BackFace,
+                    QByteArray()
+                };
+                const CameraInfo secondary = {
+                    QStringLiteral("secondary"),
+                    QGstreamerVideoInputDeviceControl::secondaryCamera(),
+                    0,
+                    QCamera::FrontFace,
+                    QByteArray()
+                };
+
+                devices.append(primary);
+                devices.append(secondary);
+
+                GstElement *camera = g_object_class_find_property(objectClass, "sensor-mount-angle")
+                        ? gst_element_factory_create(factory, 0)
+                        : 0;
+                if (camera) {
+                    if (gst_element_set_state(camera, GST_STATE_READY) != GST_STATE_CHANGE_SUCCESS) {
+                        // no-op
+                    } else for (int i = 0; i < 2; ++i) {
+                        gint orientation = 0;
+                        g_object_set(G_OBJECT(camera), "camera-device", i, NULL);
+                        g_object_get(G_OBJECT(camera), "sensor-mount-angle", &orientation, NULL);
+
+                        devices[i].orientation = (720 - orientation) % 360;
+                    }
+                    gst_element_set_state(camera, GST_STATE_NULL);
+                    gst_object_unref(GST_OBJECT(camera));
+
+                }
+            } else if (g_object_class_find_property(objectClass, "video-source")) {
+                hasVideoSource = true;
+            }
+
+            g_type_class_unref(objectClass);
+        }
+
+        if (!devices.isEmpty() || !hasVideoSource) {
+            return devices;
+        }
+    }
+
+#ifdef USE_V4L
+    QDir devDir(QStringLiteral("/dev"));
+    devDir.setFilter(QDir::System);
+
+    QFileInfoList entries = devDir.entryInfoList(QStringList()
+                << QStringLiteral("video*"));
+
+    foreach (const QFileInfo &entryInfo, entries) {
+        //qDebug() << "Try" << entryInfo.filePath();
+
+        int fd = qt_safe_open(entryInfo.filePath().toLatin1().constData(), O_RDWR );
+        if (fd == -1)
+            continue;
+
+        bool isCamera = false;
+
+        v4l2_input input;
+        memset(&input, 0, sizeof(input));
+        for (; ::ioctl(fd, VIDIOC_ENUMINPUT, &input) >= 0; ++input.index) {
+            if (input.type == V4L2_INPUT_TYPE_CAMERA || input.type == 0) {
+                isCamera = ::ioctl(fd, VIDIOC_S_INPUT, input.index) != 0;
+                break;
+            }
+        }
+
+        if (isCamera) {
+            // find out its driver "name"
+            QByteArray driver;
+            QString name;
+            struct v4l2_capability vcap;
+            memset(&vcap, 0, sizeof(struct v4l2_capability));
+
+            if (ioctl(fd, VIDIOC_QUERYCAP, &vcap) != 0) {
+                name = entryInfo.fileName();
+            } else {
+                driver = QByteArray((const char*)vcap.driver);
+                name = QString::fromUtf8((const char*)vcap.card);
+                if (name.isEmpty())
+                    name = entryInfo.fileName();
+            }
+            //qDebug() << "found camera: " << name;
+
+
+            CameraInfo device = {
+                entryInfo.absoluteFilePath(),
+                name,
+                0,
+                QCamera::UnspecifiedPosition,
+                driver
+            };
+            devices.append(device);
+        }
+        qt_safe_close(fd);
+    }
+#endif // USE_V4L
+
+    return devices;
+}
+
+QList<QByteArray> QGstUtils::cameraDevices(GstElementFactory * factory)
+{
+    QList<QByteArray> devices;
+
+    foreach (const CameraInfo &camera, enumerateCameras(factory))
+        devices.append(camera.name.toUtf8());
+
+    return devices;
+}
+
+QString QGstUtils::cameraDescription(const QString &device, GstElementFactory * factory)
+{
+    foreach (const CameraInfo &camera, enumerateCameras(factory)) {
+        if (camera.name == device)
+            return camera.description;
+    }
+    return QString();
+}
+
+QCamera::Position QGstUtils::cameraPosition(const QString &device, GstElementFactory * factory)
+{
+    foreach (const CameraInfo &camera, enumerateCameras(factory)) {
+        if (camera.name == device)
+            return camera.position;
+    }
+    return QCamera::UnspecifiedPosition;
+}
+
+int QGstUtils::cameraOrientation(const QString &device, GstElementFactory * factory)
+{
+    foreach (const CameraInfo &camera, enumerateCameras(factory)) {
+        if (camera.name == device)
+            return camera.orientation;
+    }
+    return 0;
+}
+
+QByteArray QGstUtils::cameraDriver(const QString &device, GstElementFactory *factory)
+{
+    foreach (const CameraInfo &camera, enumerateCameras(factory)) {
+        if (camera.name == device)
+            return camera.driver;
+    }
+    return QByteArray();
+}
+
+
+void qt_gst_object_ref_sink(gpointer object)
+{
+#if (GST_VERSION_MAJOR >= 0) && (GST_VERSION_MINOR >= 10) && (GST_VERSION_MICRO >= 24)
+    gst_object_ref_sink(object);
+#else
+    g_return_if_fail (GST_IS_OBJECT(object));
+
+    GST_OBJECT_LOCK(object);
+    if (G_LIKELY(GST_OBJECT_IS_FLOATING(object))) {
+        GST_OBJECT_FLAG_UNSET(object, GST_OBJECT_FLOATING);
+        GST_OBJECT_UNLOCK(object);
+    } else {
+        GST_OBJECT_UNLOCK(object);
+        gst_object_ref(object);
+    }
+#endif
+}
+
 QT_END_NAMESPACE
