@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
+** a written agreement between you and Digia. For licensing terms and
+** conditions see http://qt.digia.com/licensing. For further information
 ** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** rights. These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -184,16 +176,20 @@ void QSoundEffect::setSource(const QUrl &url)
 /*!
     \qmlproperty int QtMultimedia::SoundEffect::loops
 
-    This property provides a way to control the number of times to repeat the sound on each play().
+    This property holds the number of times the sound is played. A value of 0 or 1 means
+    the sound will be played only once; set to SoundEffect.Infinite to enable infinite looping.
 
-    Set to SoundEffect.Infinite to enable infinite looping.
+    The value can be changed while the sound effect is playing, in which case it will update
+    the remaining loops to the new value.
 */
 
 /*!
     \property QSoundEffect::loops
-    This property provides a way to control the number of times to repeat the sound on each play().
+    This property holds the number of times the sound is played. A value of 0 or 1 means
+    the sound will be played only once; set to SoundEffect.Infinite to enable infinite looping.
 
-    Set to QSoundEffect::Infinite to enable infinite looping.
+    The value can be changed while the sound effect is playing, in which case it will update
+    the remaining loops to the new value.
 */
 
 /*!
@@ -213,8 +209,14 @@ int QSoundEffect::loopCount() const
 */
 
 /*!
-    Set the total number of times to repeat playing this sound effect on each play() call to \a loopCount.
-    Pass \c QSoundEffect::Infinite to repeat until stop() is called.
+    Set the total number of times to play this sound effect to \a loopCount.
+
+    Setting the loop count to 0 or 1 means the sound effect will be played only once;
+    pass \c QSoundEffect::Infinite to repeat indefinitely. The loop count can be changed while
+    the sound effect is playing, in which case it will update the remaining loops to
+    the new \a loopCount.
+
+    \sa loopsRemaining()
 */
 void QSoundEffect::setLoopCount(int loopCount)
 {
