@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
+** a written agreement between you and Digia. For licensing terms and
+** conditions see http://qt.digia.com/licensing. For further information
 ** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** rights. These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -252,7 +244,7 @@ QIODevice* QAlsaAudioInput::start()
         delete audioSource;
 
     pullMode = false;
-    audioSource = new InputPrivate(this);
+    audioSource = new AlsaInputPrivate(this);
     audioSource->open(QIODevice::ReadOnly | QIODevice::Unbuffered);
 
     deviceState = QAudio::IdleState;
@@ -733,7 +725,7 @@ bool QAlsaAudioInput::deviceReady()
         read(0, buffer_size);
     } else {
         // emits readyRead() so user will call read() on QIODevice to get some audio data
-        InputPrivate* a = qobject_cast<InputPrivate*>(audioSource);
+        AlsaInputPrivate* a = qobject_cast<AlsaInputPrivate*>(audioSource);
         a->trigger();
     }
     bytesAvailable = checkBytesReady();
@@ -768,7 +760,7 @@ qint64 QAlsaAudioInput::elapsedUSecs() const
     if (deviceState == QAudio::StoppedState)
         return 0;
 
-    return clockStamp.elapsed()*1000;
+    return clockStamp.elapsed() * qint64(1000);
 }
 
 void QAlsaAudioInput::reset()
@@ -785,28 +777,28 @@ void QAlsaAudioInput::drain()
         snd_pcm_drain(handle);
 }
 
-InputPrivate::InputPrivate(QAlsaAudioInput* audio)
+AlsaInputPrivate::AlsaInputPrivate(QAlsaAudioInput* audio)
 {
     audioDevice = qobject_cast<QAlsaAudioInput*>(audio);
 }
 
-InputPrivate::~InputPrivate()
+AlsaInputPrivate::~AlsaInputPrivate()
 {
 }
 
-qint64 InputPrivate::readData( char* data, qint64 len)
+qint64 AlsaInputPrivate::readData( char* data, qint64 len)
 {
     return audioDevice->read(data,len);
 }
 
-qint64 InputPrivate::writeData(const char* data, qint64 len)
+qint64 AlsaInputPrivate::writeData(const char* data, qint64 len)
 {
     Q_UNUSED(data)
     Q_UNUSED(len)
     return 0;
 }
 
-void InputPrivate::trigger()
+void AlsaInputPrivate::trigger()
 {
     emit readyRead();
 }
